﻿Imports System.IO

Public Class Form1
    Private Const QuotesFilePath As String = "quotes.txt"
    Private Const QuotesOutputFilePath As String = "quotes2.txt"
    Private Const UserQuotesFilePath As String = "quotes-user.txt"
    Private Quotes As New Dictionary(Of Integer, String)()
    Private rand As New Random()
    Private isLoading As Boolean

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
        Dim fontSize As Integer = My.Settings.FontSize
        Dim fontStyle As FontStyle = CType(My.Settings.FontStyle, FontStyle)
        Dim textColor As Color = Color.FromArgb(My.Settings.TextColor)

        ApplySettings(fontSize, fontStyle, textColor)

        ToggleLoadingState(True)
        CreateQuoteFile(False)
        InitializeQuotes()
        ShowDailyQuote()
        ToggleLoadingState(False)
    End Sub

    Private Sub ToggleLoadingState(isLoadingState As Boolean)
        isLoading = isLoadingState
        Button1.Enabled = Not isLoadingState
    End Sub

    Private Sub InitializeQuotes()
        Try
            Dim outputPath As String = path.Combine(Application.StartupPath, QuotesOutputFilePath)
            If File.Exists(outputPath) Then
                Dim lines = File.ReadAllLines(outputPath)
                For Each line As String In lines
                    Dim parts = line.Split(ChrW(129))
                    Dim day As Integer
                    If parts.Length = 2 AndAlso Integer.TryParse(parts(0).Trim(), Day) Then
                        Quotes(Day) = parts(1).Trim()
                    End If
                Next
            End If
        Catch ex As Exception
            MessageBox.Show("Error loading quotes: " & ex.Message)
        End Try
    End Sub

    Private Sub ShowDailyQuote()
        Dim dayOfYear As Integer = DateTime.Now.DayOfYear
        Dim quote As String = String.Empty

        If Quotes.TryGetValue(dayOfYear, quote) Then
            If quote.Length > 80 Then
                Label1.Text = BreakQuote(quote)
                Label2.Text = quote.Substring(Label1.Text.Length)
            Else
                Label1.Text = quote
                Label2.Text = String.Empty
            End If
        Else
            Label1.Text = "No quote For today. Come back tomorrow!"
            Label2.Text = String.Empty
        End If

        CenterLabels()
    End Sub

    Private Function BreakQuote(quote As String) As String
        Dim spaceIndex As Integer = quote.LastIndexOf(" ", 80)
        Return If(spaceIndex > 0, quote.Substring(0, spaceIndex), quote)
    End Function

    Private Sub CenterLabels()
        CenterControl(Label1, -120)
        CenterControl(Label2, -95)
    End Sub

    Private Sub CenterControl(control As Control, offset As Integer)
        control.Left = (Me.ClientSize.Width - control.Width) / 2
        control.Top = (Me.ClientSize.Height / 2) - (control.Height / 2) + offset
    End Sub

    Private Sub Button1_Click(sender As Object, e As EventArgs) Handles Button1.Click
        If Not isLoading Then
            CreateQuoteFile(True)
            ShowDailyQuote()
        End If
    End Sub

    Private Sub CreateQuoteFile(isNew As Boolean)
        If isNew OrElse Not File.Exists(Path.Combine(Application.StartupPath, QuotesOutputFilePath)) Then
            isLoading = True
            Button1.Enabled = False
            Dim userQuotes As List(Of String) = If(File.Exists(Path.Combine(Application.StartupPath, "quotes-user.txt")),
                               File.ReadAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt")).ToList(),
                               New List(Of String)())
            Dim j As New HashSet(Of Integer)()
            Dim rand = New Random()

            For i As Integer = 0 To userQuotes.Count - 1
                Dim userquoteIndex As Integer
                Do
                    userquoteIndex = rand.Next(365)
                Loop While j.Contains(userquoteIndex)
                j.Add(userquoteIndex)
            Next

            Dim allQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, QuotesFilePath)).ToList()
            Using sw As New StreamWriter(Path.Combine(Application.StartupPath, QuotesOutputFilePath))
                Dim usedQuoteIndices As New HashSet(Of Integer)()
                For day As Integer = 0 To 364
                    Dim quoteToUse As String = String.Empty
                    Dim isUserQuote As Boolean = False
                    If j.Contains(day) Then
                        Dim userQuoteIndex = j.ToList().IndexOf(day)
                        If userQuoteIndex >= 0 AndAlso userQuoteIndex < userQuotes.Count Then
                            quoteToUse = userQuotes(userQuoteIndex)
                            isUserQuote = True
                        End If
                    End If
                    If Not isUserQuote Then
                        Dim quoteIndex As Integer
                        Do
                            quoteIndex = rand.Next(allQuotes.Count)
                        Loop While usedQuoteIndices.Contains(quoteIndex)
                        quoteToUse = allQuotes(quoteIndex)
                        usedQuoteIndices.Add(quoteIndex)
                    End If
                    If Not String.IsNullOrEmpty(quoteToUse) Then
                        sw.WriteLine($"{day + 1}{ChrW(129)} {quoteToUse}")
                    End If
                Next
            End Using

            InitializeQuotes()
            isLoading = False
            Button1.Enabled = True
        End If
    End Sub

    Private Sub ButtonAddQuote_Click(sender As Object, e As EventArgs) Handles ButtonAddQuote.Click
        Using addQuoteForm As New FormAddQuote()
            If addQuoteForm.ShowDialog() = DialogResult.OK Then
                AddUserQuote(addQuoteForm.NewQuote)
            End If
        End Using
    End Sub

    Private Sub AddUserQuote(quote As String)
        Try
            Using sw As New StreamWriter(Path.Combine(Application.StartupPath, "quotes-user.txt"), True)
                sw.WriteLine(quote)
            End Using
            ShowDailyQuote()
            MessageBox.Show("Quote added successfully!")
        Catch ex As Exception
            MessageBox.Show("Error adding quote: " & ex.Message)
        End Try
    End Sub

    Private Sub ButtonEditQuote_Click(sender As Object, e As EventArgs) Handles ButtonEditQuote.Click
        Using editQuoteForm As New FormEditQuote()
            If editQuoteForm.ShowDialog() = DialogResult.OK Then
                EditUserQuote(editQuoteForm.SelectedQuote)
            End If
        End Using
    End Sub

    Private Sub EditUserQuote(newQuote As String)
        Try
            Dim userQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt")).ToList()

            userQuotes(userQuotes.IndexOf(EditedQuote)) = newQuote

            File.WriteAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt"), userQuotes)

            CreateQuoteFile(True)
            ShowDailyQuote()
            MessageBox.Show("Quote edited successfully!")
        Catch ex As Exception
            MessageBox.Show("Error editing quote: " & ex.Message)
        End Try
    End Sub

    Private Sub ButtonDeleteQuote_Click(sender As Object, e As EventArgs) Handles ButtonDeleteQuote.Click
        Using deleteQuoteForm As New FormDeleteQuote()
            If deleteQuoteForm.ShowDialog() = DialogResult.OK Then
                DeleteUserQuote(deleteQuoteForm.SelectedQuote)
            End If
        End Using
    End Sub

    Private Sub DeleteUserQuote(quoteToDelete As String)
        Try
            Dim userQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt")).ToList()

            If userQuotes.Remove(quoteToDelete) Then
                File.WriteAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt"), userQuotes)

                CreateQuoteFile(True)
                ShowDailyQuote()
                MessageBox.Show("Quote deleted successfully!")
            Else
                MessageBox.Show("Quote not found!")
            End If
        Catch ex As Exception
            MessageBox.Show("Error deleting quote: " & ex.Message)
        End Try
    End Sub

    Private Sub ButtonRandomQuote_Click(sender As Object, e As EventArgs) Handles ButtonRandomQuote.Click
        ShowRandomQuote()
    End Sub

    Private Sub ShowRandomQuote()
        Dim allUserQuotes As List(Of String) = If(File.Exists(Path.Combine(Application.StartupPath, "quotes-user.txt")),
                                                    File.ReadAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt")).ToList(),
                                                    New List(Of String)())

        Dim allQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, QuotesFilePath)).ToList()

        Dim combinedQuotes As New List(Of String)(allQuotes)
        combinedQuotes.AddRange(allUserQuotes)

        If combinedQuotes.Count > 0 Then
            Dim randomIndex As Integer = rand.Next(combinedQuotes.Count)
            DisplayQuote(combinedQuotes(randomIndex))
        Else
            Label1.Text = "No quotes available!"
            Label2.Text = String.Empty
        End If

        CenterLabels()
    End Sub

    Private Sub DisplayQuote(quote As String)
        If quote.Length > 85 Then
            Label1.Text = BreakQuote(quote)
            Label2.Text = quote.Substring(Label1.Text.Length)
        Else
            Label1.Text = quote
            Label2.Text = String.Empty
        End If
    End Sub

    Private Sub ButtonSettings_Click(sender As Object, e As EventArgs) Handles ButtonSettings.Click
        Using settingsForm As New FormSettings()
            If settingsForm.ShowDialog() = DialogResult.OK Then
                ApplySettings(settingsForm.SelectedFontSize, settingsForm.SelectedFontStyle, settingsForm.SelectedTextColor)
            End If
        End Using
    End Sub

    Private Sub ApplySettings(fontSize As Integer, fontStyle As FontStyle, textColor As Color)
        Label1.Font = New Font(Label1.Font.FontFamily, fontSize, fontStyle)
        Label1.ForeColor = textColor

        Label2.Font = New Font(Label2.Font.FontFamily, fontSize, fontStyle)
        Label2.ForeColor = textColor

        ShowDailyQuote()
    End Sub

    Private Sub ButtonSaveQuotes_Click(sender As Object, e As EventArgs) Handles ButtonSaveQuotes.Click
        Using saveFileDialog As New SaveFileDialog()
            saveFileDialog.Filter = "Text Files|*.txt|All Files|*.*"
            saveFileDialog.Title = "Save Quotes File"
            saveFileDialog.FileName = "AllQuotes.txt"

            If saveFileDialog.ShowDialog() = DialogResult.OK Then
                Try
                    SaveAllQuotesToFile(saveFileDialog.FileName)
                    MessageBox.Show("Quotes saved successfully!")
                Catch ex As Exception
                    MessageBox.Show("Error saving quotes: " & ex.Message)
                End Try
            End If
        End Using
    End Sub

    Private Sub SaveAllQuotesToFile(filePath As String)
        Dim allQuotes As New List(Of String)()

        Dim predefinedQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, QuotesFilePath)).ToList()
        allQuotes.AddRange(predefinedQuotes)

        If File.Exists(Path.Combine(Application.StartupPath, "quotes-user.txt")) Then
            Dim userQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt")).ToList()
            allQuotes.AddRange(userQuotes)
        End If

        File.WriteAllLines(filePath, allQuotes)
    End Sub

    Private Sub ButtonSearch_Click(sender As Object, e As EventArgs) Handles ButtonSearch.Click
        Dim searchTerm As String = TextBoxSearch.Text.Trim()
        If String.IsNullOrEmpty(searchTerm) Then
            MessageBox.Show("Please enter a search term.")
            Return
        End If
        PerformSearch(searchTerm.ToLower())
    End Sub

    Private Sub PerformSearch(searchTerm As String)
        RichTextBoxResults.Clear()
        Dim found As Boolean = False

        Dim predefinedQuotes As List(Of String) = File.ReadAllLines(Path.Combine(Application.StartupPath, QuotesFilePath)).ToList()

        Dim userQuotes As List(Of String) = If(File.Exists(Path.Combine(Application.StartupPath, "quotes-user.txt")),
                                            File.ReadAllLines(Path.Combine(Application.StartupPath, "quotes-user.txt")).ToList(),
                                            New List(Of String)())

        Dim allQuotes As New List(Of String)()
        allQuotes.AddRange(predefinedQuotes)
        allQuotes.AddRange(userQuotes)

        For Each quote As String In allQuotes
            If quote.ToLower().Contains(searchTerm) Then
                found = True
                HighlightAndAddQuote(quote, searchTerm)
            End If
        Next

        If Not found Then
            MessageBox.Show("No quotes found matching your search term.")
        End If
    End Sub

    Private Sub btnClear_Click(sender As Object, e As EventArgs) Handles btnClear.Click
        TextBoxSearch.Text = ""
        RichTextBoxResults.Text = ""
    End Sub

    Private Sub HighlightAndAddQuote(quote As String, searchTerm As String)
        Dim originalLength As Integer = RichTextBoxResults.TextLength

        RichTextBoxResults.AppendText(quote & Environment.NewLine)

        Dim currentQuoteLength As Integer = quote.Length
        Dim startIndex As Integer = originalLength

        While startIndex < originalLength + currentQuoteLength
            startIndex = RichTextBoxResults.Find(searchTerm, startIndex, RichTextBoxResults.TextLength, RichTextBoxFinds.None)
            If startIndex = -1 Then Exit While

            RichTextBoxResults.Select(startIndex, searchTerm.Length)
            RichTextBoxResults.SelectionColor = Color.Red

            startIndex += searchTerm.Length
        End While

        RichTextBoxResults.Select(RichTextBoxResults.TextLength, 0)
        RichTextBoxResults.SelectionColor = Color.Black
    End Sub

End Class
